"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.retrieveCachedScript = exports.removeScriptUpdater = exports.addScriptUpdater = exports.updateCachedScript = exports.getFromWindowCache = void 0;
if (!Object.prototype.hasOwnProperty.call(window, "__loadedScripts")) {
    Object.defineProperty(window, "__loadedScripts", {
        set: function (value) { return (window.__loadedScriptsInternal = value); },
        get: function () {
            return (window.__loadedScriptsInternal = window.__loadedScriptsInternal || {});
        },
    });
}
if (!Object.prototype.hasOwnProperty.call(window, "__loadedScriptsUpdaters")) {
    Object.defineProperty(window, "__loadedScriptsUpdaters", {
        set: function (value) {
            return (window.__loadedScriptsUpdatersInternal = value);
        },
        get: function () {
            return (window.__loadedScriptsUpdatersInternal =
                window.__loadedScriptsUpdatersInternal || {});
        },
    });
}
var defaultCachedScript = {
    loading: true,
    failed: false,
    scriptCreated: false,
};
var getCachedScriptUpdaters = function (url) {
    return (window.__loadedScriptsUpdaters[url] =
        window.__loadedScriptsUpdaters[url] || []);
};
function getFromWindowCache(url) {
    return (window.__loadedScripts[url] = window.__loadedScripts[url] || __assign(__assign({}, defaultCachedScript), { url: url }));
}
exports.getFromWindowCache = getFromWindowCache;
function updateCachedScript(url, updatedScript) {
    var newScript = (window.__loadedScripts[url] = __assign(__assign({}, getFromWindowCache(url)), updatedScript));
    getCachedScriptUpdaters(url).forEach(function (updater) { return updater(newScript); });
    return newScript;
}
exports.updateCachedScript = updateCachedScript;
function addScriptUpdater(url, updater) {
    window.__loadedScriptsUpdaters[url] = __spreadArray(__spreadArray([], __read(getCachedScriptUpdaters(url)), false), [
        updater,
    ], false);
}
exports.addScriptUpdater = addScriptUpdater;
function removeScriptUpdater(url, updater) {
    window.__loadedScriptsUpdaters[url] = getCachedScriptUpdaters(url).filter(function (currentUpdater) { return currentUpdater !== updater; });
}
exports.removeScriptUpdater = removeScriptUpdater;
var getNewScript = function (source, scriptAttributes) {
    var newScript = document.createElement("script");
    newScript.async = true;
    newScript.setAttribute("src", source);
    Object.entries(scriptAttributes !== null && scriptAttributes !== void 0 ? scriptAttributes : {}).forEach(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        return newScript.setAttribute(key, value);
    });
    return newScript;
};
var setupListeners = function (scriptRef, source) {
    var removeListeners = function () {
        scriptRef.removeEventListener("load", loadEvent);
        scriptRef.removeEventListener("error", errorEvent);
    };
    var generateScriptEventListener = function (getResultingCachedScript) {
        return function (ev) {
            updateCachedScript(source, getResultingCachedScript(ev));
            removeListeners();
        };
    };
    var loadEvent = generateScriptEventListener(function () { return ({
        loading: false,
        failed: false,
    }); });
    var errorEvent = generateScriptEventListener(function (err) { return ({
        loading: false,
        failed: true,
        failureEvent: err,
    }); });
    scriptRef.addEventListener("load", loadEvent);
    scriptRef.addEventListener("error", errorEvent);
};
function retrieveCachedScript(source, scriptAttributes) {
    var scriptQuery = Object.entries(scriptAttributes !== null && scriptAttributes !== void 0 ? scriptAttributes : {}).reduce(function (acc, _a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        return "".concat(acc, "[").concat(key, "=\"").concat(value, "\"]");
    }, "");
    return document.querySelector("script[src=\"".concat(source, "\"]").concat(scriptQuery))
        ? (function () {
            var cachedScriptInfo = getFromWindowCache(source);
            // if we did not create the script, assume it has loaded
            if (!cachedScriptInfo.scriptCreated)
                return updateCachedScript(source, {
                    loading: false,
                    failed: false,
                });
            return cachedScriptInfo;
        })()
        : (function () {
            var newRef = getNewScript(source, scriptAttributes);
            setupListeners(newRef, source);
            document.body.appendChild(newRef);
            return updateCachedScript(source, { scriptCreated: true });
        })();
}
exports.retrieveCachedScript = retrieveCachedScript;
