var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useCallback, useContext, useMemo, createContext } from "react";
import useScriptLoader from "./hooks/useScriptLoader";
import useSafeState from "./hooks/useSafeState";
const ScriptLoaderContext = createContext({
    loaderStatus: "loading",
});
const noop = () => {
    //intentionally empty
};
const ScriptLoader = (_a) => {
    var { source, onSuccess = noop, onError = noop } = _a, props = __rest(_a, ["source", "onSuccess", "onError"]);
    const [isSucceeded, setIsSucceeded] = useSafeState(false);
    const [isFailed, setIsFailed] = useSafeState(false);
    const loaderStatus = useMemo(() => {
        if (isSucceeded)
            return "succeeded";
        if (isFailed)
            return "failed";
        return "loading";
    }, [isSucceeded, isFailed]);
    const onLoaderSuccess = useCallback(() => {
        setIsSucceeded(true);
        setIsFailed(false);
        onSuccess();
    }, [onSuccess, setIsSucceeded, setIsFailed]);
    const onLoaderFailure = useCallback((err) => {
        setIsFailed(true);
        setIsSucceeded(false);
        onError(err);
    }, [onError, setIsSucceeded, setIsFailed]);
    useScriptLoader({
        source,
        onSuccess: onLoaderSuccess,
        onFailure: onLoaderFailure,
    });
    return React.createElement(ScriptLoaderContext.Provider, Object.assign({ value: { loaderStatus } }, props));
};
const Success = ({ children }) => {
    const { loaderStatus } = useContext(ScriptLoaderContext);
    return loaderStatus === "succeeded" ? React.createElement(React.Fragment, null, children) : React.createElement(React.Fragment, null);
};
const Failed = ({ children }) => {
    const { loaderStatus } = useContext(ScriptLoaderContext);
    return loaderStatus === "failed" ? React.createElement(React.Fragment, null, children) : React.createElement(React.Fragment, null);
};
const Loading = ({ children }) => {
    const { loaderStatus } = useContext(ScriptLoaderContext);
    return loaderStatus === "loading" ? React.createElement(React.Fragment, null, children) : React.createElement(React.Fragment, null);
};
ScriptLoader.Success = Success;
ScriptLoader.Failed = Failed;
ScriptLoader.Loading = Loading;
export default ScriptLoader;
