var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useCallback, useEffect, useRef, useState } from "react";
import { waitForScript } from "../scriptloader-support";
export default (function useScriptLoader(config) {
    const { source, onSuccess, onFailure = () => {
        //noop
    }, scriptAttributes: configAttributes, } = config;
    const [scriptAttributes, setScriptAttributes] = useState(configAttributes);
    const isMounted = useRef(true);
    useEffect(() => () => {
        isMounted.current = false;
    }, []);
    const successFunc = useCallback(() => isMounted.current && onSuccess(), [onSuccess]);
    const errorFunc = useCallback((err) => isMounted.current && onFailure(err), [onFailure]);
    useEffect(() => {
        setScriptAttributes((oldAttrs) => {
            if (Boolean(oldAttrs) !== Boolean(configAttributes)) {
                return configAttributes;
            }
            if (!oldAttrs) {
                return oldAttrs;
            }
            const oldEntries = Object.entries(oldAttrs);
            const newEntries = Object.entries(configAttributes);
            if (oldEntries.length !== newEntries.length) {
                return configAttributes;
            }
            if (oldEntries.some(([key, value]) => configAttributes[key] !== value)) {
                return configAttributes;
            }
            return oldAttrs;
        });
    }, [configAttributes]);
    useEffect(() => {
        const waitForSource = () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield waitForScript(source, scriptAttributes);
                successFunc();
            }
            catch (err) {
                errorFunc(err);
            }
        });
        void waitForSource();
    }, [source, successFunc, errorFunc, scriptAttributes]);
});
