if (!Object.prototype.hasOwnProperty.call(window, "__loadedScripts")) {
    Object.defineProperty(window, "__loadedScripts", {
        set: (value) => (window.__loadedScriptsInternal = value),
        get: () => (window.__loadedScriptsInternal = window.__loadedScriptsInternal || {}),
    });
}
if (!Object.prototype.hasOwnProperty.call(window, "__loadedScriptsUpdaters")) {
    Object.defineProperty(window, "__loadedScriptsUpdaters", {
        set: (value) => (window.__loadedScriptsUpdatersInternal = value),
        get: () => (window.__loadedScriptsUpdatersInternal =
            window.__loadedScriptsUpdatersInternal || {}),
    });
}
const defaultCachedScript = {
    loading: true,
    failed: false,
    scriptCreated: false,
};
const getCachedScriptUpdaters = (url) => (window.__loadedScriptsUpdaters[url] =
    window.__loadedScriptsUpdaters[url] || []);
export function getFromWindowCache(url) {
    return (window.__loadedScripts[url] = window.__loadedScripts[url] || Object.assign(Object.assign({}, defaultCachedScript), { url }));
}
export function updateCachedScript(url, updatedScript) {
    const newScript = (window.__loadedScripts[url] = Object.assign(Object.assign({}, getFromWindowCache(url)), updatedScript));
    getCachedScriptUpdaters(url).forEach((updater) => updater(newScript));
    return newScript;
}
export function addScriptUpdater(url, updater) {
    window.__loadedScriptsUpdaters[url] = [
        ...getCachedScriptUpdaters(url),
        updater,
    ];
}
export function removeScriptUpdater(url, updater) {
    window.__loadedScriptsUpdaters[url] = getCachedScriptUpdaters(url).filter((currentUpdater) => currentUpdater !== updater);
}
const getNewScript = (source, scriptAttributes) => {
    const newScript = document.createElement("script");
    newScript.async = true;
    newScript.setAttribute("src", source);
    Object.entries(scriptAttributes !== null && scriptAttributes !== void 0 ? scriptAttributes : {}).forEach(([key, value]) => newScript.setAttribute(key, value));
    return newScript;
};
const setupListeners = (scriptRef, source) => {
    const removeListeners = () => {
        scriptRef.removeEventListener("load", loadEvent);
        scriptRef.removeEventListener("error", errorEvent);
    };
    const generateScriptEventListener = (getResultingCachedScript) => (ev) => {
        updateCachedScript(source, getResultingCachedScript(ev));
        removeListeners();
    };
    const loadEvent = generateScriptEventListener(() => ({
        loading: false,
        failed: false,
    }));
    const errorEvent = generateScriptEventListener((err) => ({
        loading: false,
        failed: true,
        failureEvent: err,
    }));
    scriptRef.addEventListener("load", loadEvent);
    scriptRef.addEventListener("error", errorEvent);
};
export function retrieveCachedScript(source, scriptAttributes) {
    const scriptQuery = Object.entries(scriptAttributes !== null && scriptAttributes !== void 0 ? scriptAttributes : {}).reduce((acc, [key, value]) => `${acc}[${key}="${value}"]`, "");
    return document.querySelector(`script[src="${source}"]${scriptQuery}`)
        ? (() => {
            const cachedScriptInfo = getFromWindowCache(source);
            // if we did not create the script, assume it has loaded
            if (!cachedScriptInfo.scriptCreated)
                return updateCachedScript(source, {
                    loading: false,
                    failed: false,
                });
            return cachedScriptInfo;
        })()
        : (() => {
            const newRef = getNewScript(source, scriptAttributes);
            setupListeners(newRef, source);
            document.body.appendChild(newRef);
            return updateCachedScript(source, { scriptCreated: true });
        })();
}
